#ifndef _CHAMPLAIN_PERL_H_

#include <clutter-perl.h>

#include <champlain/champlain.h>

#ifdef CHAMPLAINPERL_MEMPHIS
#include <memphis-perl.h>
#include <champlain/champlain-memphis.h>
#endif

#include "champlain-autogen.h"

#endif /* _CHAMPLAIN_PERL_H_ */
